/* expdbflib - library to export data from dbf files 
 * Copyright (C) 1997-1999 Zbigniew Michal Kempczynski
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#if !defined (__DBF_H)
#define __DBF_H

#include "defs.h"
#include <stdio.h>
#include "global.h"

#define DBF_END_OF_BLOCKS 0x0d //znak konczacy wystepowanie blokow w dbf

typedef struct
{
  short year;
  char month;
  char day;
} DBFdate;

/*		    NISKOPOZIOMOWE STRUKTURY I FUNKCJE			*/

/*----------------------------------------------------------------------*/
/*			Definicja typu DBFheader			*/
/*----------------------------------------------------------------------*/
typedef struct
{
 char id;	    // identyfikator bazy
 char date[3];	    // data w formacie rok (dwie ost. cyfry), miesiac, dzien
 long recnum;	   // liczba rekordow w bazie
 short firstrecpos;  // offset pierwszego rekordu w stosunku do poczatku pliku
 short reclen;	     // dlugosc rekordu
 char reserved[20]; // zarezerwowane
} DBFheader;

/*----------------------------------------------------------------------*/
/*			Definicja typu DBFblock 			*/
/*----------------------------------------------------------------------*/
typedef struct
{
  char fieldname[11]; //nazwa rekordu
  char fieldtype;     /* typ pola : C(har) 43h	  | D(ata) 44h /
				    L(ogical) 4ch | M(emo) 4dh /
				    N(umeric) 4eh |	      */
  char reserved[4];   //zarezerwowane
  char fieldlen[2];   /* dla numerycznych 1 bajt = dlugosc pola
					  2 bajt = ilosc miejsc po przecinku
			 dla innych typow 1 bajt = dlugosc pola
					  2 bajt = nieuzywany		  */
  char notused[14];   // nieuzywane
} DBFblock;

/*----------------------------------------------------------------------*/
/*	  Standardowe funkcje operujace na DBFheader i DBFblock 	*/
/*----------------------------------------------------------------------*/
/* NISKOPOZIOMOWE FUNKCJE */
			      // na DBFheader
int DBF_fill_header(FILE *dbfdes, DBFheader *dbfh);
void DBF_show_header(DBFheader *dbfh);
char DBF_get_id(DBFheader *dbfh);
DBFdate DBF_get_date(DBFheader *dbfh);
void DBF_show_date(DBFdate *dbfdate);
long DBF_get_recnum(DBFheader *dbfh);
word DBF_get_firstrecpos(DBFheader *dbfh);
word DBF_get_reclen(DBFheader *dbfh);

			     // na DBFblock
int DBF_set_block(FILE *dbfdes, int blocknum);
int DBF_read_block(FILE *dbfdes, DBFblock *dbfb);
void DBF_show_block(DBFblock *dbfb);
int DBF_is_end_of_blocks(DBFblock *dbfb);

/* NOTE : by uzywac powyzszych funkcji nalezy miec otwarty plik *.dbf
	  i przekazac tylko jego deskryptor - by uniknac ciaglego otwierania
	  i zamykania pliku */


/*		      SREDNIOPOZIOMOWE STRUKTURY I FUNKCJE		*/

/*----------------------------------------------------------------------*/
/* DBFdesheader - struktura zawierajaca deskryptor i naglowek dbf	*/
/*----------------------------------------------------------------------*/

typedef struct
{
  char dbfname[255]; //nazwa pliku do otwarcia
  FILE *dbfdes;      //deskryptor pliku
  DBFheader dbfh;    //naglowek pliku dbf
} DBFdescheader;

typedef struct
{
  short blocknum;      //liczba blokow w pliku dbf
  DBFblock *dbfbtab;   //tablica blokow
  short *dbffieldpos;  //pozycja pola w rekordzie
} DBFblocktree;

typedef struct
{
  int actrec;	     // aktulnie wczytywany rekord
  void *rec;	      // tu jest rekord
} DBFrecord;

typedef struct
{
  short fieldlen;
  char *field;
} DBFfield;


/* SREDNIOPOZIOMOWE FUNKCJE */
int DBF_open_dbffile(DBFdescheader *dbfdh, char *dbfname);
void DBF_show_dbffile(DBFdescheader *dbfdh);
int DBF_close_dbffile(DBFdescheader *dbfdh);

void DBF_create_block_tree(DBFblocktree *dbfbt, DBFdescheader *dbfdh);
void DBF_destroy_block_tree(DBFblocktree *dbfbt);
void DBF_show_block_tree(DBFblocktree *dbfbt);
int DBF_count_blocks(DBFdescheader *dbfdh);

void DBF_allocate_record(DBFdescheader *dbfdh, DBFrecord *dbfr);
void DBF_set_record(DBFdescheader *dbfdh, DBFrecord *dbfr, long rec);
void DBF_release_record(DBFrecord *dbfr);
int DBF_read_record(DBFdescheader *dbfdh, DBFrecord *dbfr);
int DBF_get_field_len(DBFblocktree *dbfbt, int fieldnum);
int DBF_get_field_num(DBFblocktree *dbfbt, char *fieldname);
void DBF_show_field(DBFblocktree *dbfbt, DBFrecord *rec, int fieldnum);
void DBF_show_field_by_fieldname(DBFblocktree *dbfbt, DBFrecord *rec, char *fieldname);
DBFfield DBF_get_field_from_record(DBFblocktree *dbfbt, DBFrecord *rec, int fieldnum);
DBFfield DBF_get_field_from_record_by_fieldname(DBFblocktree *dbfbt, DBFrecord *rec, char *fieldname);


/*			  WYSOKOPOZIOMOWA KLASA 			*/

class DBF
{
 private:
  DBFdescheader dbfdh;	  //deskryptor pliku + naglowek dbf
  DBFblocktree dbfbt;	  //tablica blokow
  DBFrecord dbfr;	  //rekord
  DBFfield dbff;	  //pole
 public:
  DBF(char *dbfname);		 //konstruktor
  ~DBF(void);			 //destruktor
  long DBF_get_recnum(void);	 //funkcja zwraca liczbe rekordow
  void DBF_read_record(long rec);//wczytuje dany rekord
  void DBF_get_field_by_num(int fieldnum);
  void DBF_get_field_by_name(char *fieldname);
  void DBF_show_field(void);
  char *DBF_get_field_addr(void);
  void DBF_show_header(void);	 //pokazanie naglowka pliku
  void DBF_show_fields(void);	 //pokazanie wszystkich pol i informacji o nich
  void DBF_show_field_by_num(int fieldnum); //pokazanie pola o numerze fieldnum
  void DBF_show_field_by_name(char *fieldname); //j.w. ale wg fieldname
};



#endif

